<?php

namespace App\Http\Controllers;

use App\Events\SendMessages;
use App\Models\Chat;
use App\Models\User;
use App\Notifications\MessageNotification;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Pusher\Pusher;
use Pusher\PusherException;

class Chatcontroller extends Controller
{
    public function index(User $user)
    {
        $messages = Chat::where(function ($query) use ($user) {
            $query->where('sender_id', Auth::guard('web')->user()->id)
                ->where('receiver_id', $user->id);
        })->orWhere(function ($query) use ($user) {
            $query->where('sender_id', $user->id)
                ->where('receiver_id', Auth::guard('web')->user()->id);
        })->orderBy('created_at', 'asc')->get();

        return view('index', compact('user', 'messages'));
    }

    public function send(Request $request)
    {
        $message = Chat::create([
            'sender_id' => Auth::guard('web')->user()->id,
            'receiver_id' => $request->receiver_id,
            'message' => $request->message,
        ]);

        // Broadcast the event to Pusher
        $pusher = new Pusher(
            env('PUSHER_APP_KEY'),
            env('PUSHER_APP_SECRET'),
            env('PUSHER_APP_ID'),
            [
                'cluster' => env('PUSHER_APP_CLUSTER'),
                'useTLS' => true,
            ]
        );

        $pusher->trigger('private-chat-' . $request->receiver_id, 'message-sent', [
            'message' => $message->message,
            'sender_id' => $message->sender_id,
        ]);

        return response()->json(['success' => true]);
    }

    public function user_chat($userid)
    {
        $check = WebsiteController::verifySubscription();
        if ($check == 0) {
            return redirect()->route('subscriptions')->with('error', 'please purchase subscription first.');
        }
        if($userid==Auth::guard('web')->user()->id)
        {
            return redirect()->route('user.chat');
        }
        else
        {
            $user = User::findOrFail($userid);
            Chat::where('sender_id', $user->id)->where('receiver_id', Auth::guard('web')->user()->id)->where('seen', 0)->update(['seen' => 1]);
            return view("website.chat", compact('user'));
        }
    }

    public function send_message(Request $request)
    {
        $validated = $request->validate([
            'message' => 'required|string',
            'receiver_id' => 'required|exists:users,id',
        ]);

        $message = Chat::create([
            'sender_id' => Auth::guard('web')->user()->id,
            'receiver_id' => $validated['receiver_id'],
            'message' => $validated['message'],
        ]);

        broadcast(new SendMessages($message))->toOthers();

        return response()->json([
            'success' => true,
            'message' => $message,
        ]);
    }

    public function send_message_from_host(Request $request)
    {
        $validated = $request->validate([
            'message' => 'required|string',
            'receiver_id' => 'required|exists:users,id',
        ]);

        $message = Chat::create([
            'sender_id' => Auth::guard('host')->user()->id,
            'receiver_id' => $validated['receiver_id'],
            'message' => $validated['message'],
        ]);

        broadcast(new SendMessages($message))->toOthers();

        return response()->json([
            'success' => true,
            'message' => $message,
        ]);
    }

    public function fetchMessages(Request $request)
    {
        $receiverid = $request->receiver_id;
        $senderid = Auth::guard('web')->user()->id;
        $user = User::findOrFail(Auth::guard('web')->user()->id);

        Chat::where('sender_id', $receiverid)->where('receiver_id', $senderid)->where('seen', "0")->update(['seen' => "1"]);

        if (!$user) {
            return response()->json([
                'error' => 'User not found. You must be logged in to access messages.'
            ], 404);
        }

        // Fetch messages between the admin and the specified seller
        $messages = Chat::where(function ($query) use ($senderid, $receiverid) {
            $query->where('sender_id', $senderid)
                ->where('receiver_id', $receiverid);
        })->orWhere(function ($query) use ($senderid, $receiverid) {
            $query->where('sender_id', $receiverid)
                ->where('receiver_id', $senderid);
        })->with(['sender','reciever'])->orderBy('created_at', 'asc')->get();
        return response()->json($messages);
    }

    public function loadChatUsers(Request $request)
    {
        $userId = Auth::guard('web')->user()->id;

        // Fetch users who have chatted with the current user
        $users = Chat::where('sender_id', $userId)
            ->orWhere('receiver_id', $userId)
            ->with(['sender', 'reciever'])
            ->get()
            ->groupBy(function ($message) use ($userId) {
                return $message->sender_id == $userId ? $message->receiver_id : $message->sender_id;
            });

        // Transform the data to include relevant user information
        $chatUsers = [];
        foreach ($users as $otherUserId => $messages) {
            $unseenMessagesCount = $messages->filter(function ($message) use ($userId) {
                return $message->receiver_id == $userId && $message->seen == 0; // Check for unseen messages
            })->count();
            $otherUser = $messages->first()->sender_id == $userId
                ? $messages->first()->reciever
                : $messages->first()->sender;

            $chatUsers[] = [
                'id' => $otherUser->id,
                'name' => $otherUser->name,
                'image' => $otherUser->image,
                'country' => $otherUser->country,
                'unseen' => $unseenMessagesCount,
            ];
        }

        // Sort users by unseen messages count in descending order
        usort($chatUsers, function ($a, $b) {
            return $b['unseen'] <=> $a['unseen'];
        });

        return response()->json($chatUsers);
    }

    public function loadChatUsersforhost(Request $request)
    {
        $userId = Auth::guard('host')->user()->id;

        // Fetch users who have chatted with the current user
        $users = Chat::where('sender_id', $userId)
            ->orWhere('receiver_id', $userId)
            ->with(['sender', 'reciever'])
            ->get()
            ->groupBy(function ($message) use ($userId) {
                return $message->sender_id == $userId ? $message->receiver_id : $message->sender_id;
            });

        // Transform the data to include relevant user information
        $chatUsers = [];
        foreach ($users as $otherUserId => $messages) {
            $unseenMessagesCount = $messages->filter(function ($message) use ($userId) {
                return $message->receiver_id == $userId && $message->seen == 0; // Check for unseen messages
            })->count();
            $otherUser = $messages->first()->sender_id == $userId
                ? $messages->first()->reciever
                : $messages->first()->sender;

            $chatUsers[] = [
                'id' => $otherUser->id,
                'name' => $otherUser->name,
                'image' => $otherUser->image,
                'country' => $otherUser->country,
                'unseen' => $unseenMessagesCount,
            ];
        }

        // Sort users by unseen messages count in descending order
        usort($chatUsers, function ($a, $b) {
            return $b['unseen'] <=> $a['unseen'];
        });

        return response()->json($chatUsers);
    }

    public function fetchhostMessages(Request $request)
    {
        $receiverid = $request->receiver_id;
        $senderid = Auth::guard('host')->user()->id;
        $user = User::findOrFail(Auth::guard('host')->user()->id);

        if (!$user) {
            return response()->json([
                'error' => 'User not found. You must be logged in to access messages.'
            ], 404);
        }

        Chat::where('sender_id', $receiverid)->where('receiver_id', $senderid)->where('seen', "0")->update(['seen' => "1"]); // Mark as seen

        // Fetch messages between the admin and the specified seller
        $messages = Chat::where(function ($query) use ($senderid, $receiverid) {
            $query->where('sender_id', $senderid)
                ->where('receiver_id', $receiverid);
        })->orWhere(function ($query) use ($senderid, $receiverid) {
            $query->where('sender_id', $receiverid)
                ->where('receiver_id', $senderid);
        })->with(['sender','reciever'])->orderBy('created_at', 'asc')->get();
        return response()->json($messages);
    }


}
